/*
 * i2c_interface.c
 *
 *  Created on: 2018312
 *      Author: admin
 */
#include <cyu3system.h>
#include <cyu3os.h>
#include <cyu3dma.h>
#include <cyu3error.h>
#include <cyu3uart.h>
#include <cyu3i2c.h>
#include <cyu3types.h>
#include <cyu3gpio.h>
#include <cyu3utils.h>
#include "i2c_interface.h"

static int i2c_interface_write(void *p, uint32_t addr, uint32_t value);
static int i2c_interface_read(void *p, uint32_t addr, uint32_t *pValue);
static int i2c_interface_write_burst(void *p, uint32_t addr, uint8_t *pBuf, uint32_t size);
static int i2c_interface_read_burst(void *p, uint32_t addr, uint8_t *pBuf, uint32_t size);
static int i2c_interface_generate_write_preamble(I2COBJ_t *pObj,  uint32_t addr, CyU3PI2cPreamble_t *p_preamble);
static int i2c_interface_generate_read_preamble(I2COBJ_t *pObj, uint32_t addr, CyU3PI2cPreamble_t *p_preamble);


int i2c_interface_init(I2COBJ_t *pObj, uint8_t slave_addr, uint8_t addr_len, uint8_t data_len)
{
	if((!pObj) || (addr_len > 4) || (data_len > 4))
	{
		return -1;
	}

	pObj->slave_addr = slave_addr;
	pObj->addr_len = addr_len;
	pObj->data_len = data_len;

	pObj->Write = i2c_interface_write;
	pObj->Read = i2c_interface_read;
	pObj->WriteBurst = i2c_interface_write_burst;
	pObj->ReadBurst = i2c_interface_read_burst;

	return 0;
}

static int i2c_interface_write(void *p, uint32_t addr, uint32_t value)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
	uint8_t buf[4];
	I2COBJ_t *pObj = (I2COBJ_t *)p;
	int count = pObj->data_len;

	i2c_interface_generate_write_preamble(pObj, addr, &preamble);//һȵͷ

	if(pObj->data_len == 1)  //ͬݳȣӦͬȡݷʽ
	{
		buf[0] = UINT32BYTE0(value);
	}
	else if(pObj->data_len == 2)
	{
		buf[0] = UINT32BYTE1(value);
		buf[1] = UINT32BYTE0(value);
	}
	else if (pObj->data_len == 3)
	{
		buf[0] = UINT32BYTE2(value);
		buf[1] = UINT32BYTE1(value);
		buf[2] = UINT32BYTE0(value);
	}
	else
	{
		buf[0] = UINT32BYTE3(value);
		buf[1] = UINT32BYTE2(value);
		buf[2] = UINT32BYTE1(value);
		buf[3] = UINT32BYTE0(value);
	}
    apiRetStatus = CyU3PI2cTransmitBytes (&preamble, buf, count, 0);
	if(apiRetStatus == CY_U3P_SUCCESS)
	{
		//apiRetStatus = CyU3PI2cWaitForAck(&preamble, 0);
	}

    return apiRetStatus;
}

static int i2c_interface_read(void *p, uint32_t addr, uint32_t *pValue)
{
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
	uint8_t buf[4];
	I2COBJ_t *pObj = (I2COBJ_t *)p;

	int count = pObj->data_len;
	uint32_t value = 0xff;

	i2c_interface_generate_read_preamble(pObj, addr, &preamble);

    apiRetStatus = CyU3PI2cReceiveBytes (&preamble, buf, count, 3);

    if(apiRetStatus == CY_U3P_SUCCESS)
	{
		if(pObj->data_len == 1)
		{
			value = (uint32_t)buf[0];
		}
		else if(pObj->data_len == 2)
		{
			value = ((uint32_t)buf[0] << 8) + (uint32_t)buf[1];
		}
		else if (pObj->data_len == 3)
		{
			value = ((uint32_t)buf[0] << 16) + ((uint32_t)buf[1] << 8)
			        + ((uint32_t)buf[2]);
		}
		else
		{
			value = ((uint32_t)buf[0] << 24) + ((uint32_t)buf[1] << 16)
			        + ((uint32_t)buf[2] << 8) + ((uint32_t)buf[3]);
		}
		*pValue = value;
	}

    CyU3PThreadSleep (1);

    return apiRetStatus;
}

static int i2c_interface_write_burst(void *p, uint32_t addr, uint8_t *pBuf, uint32_t size)
{
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

	CyU3PI2cPreamble_t preamble;
	I2COBJ_t *pObj = (I2COBJ_t *)p;

	i2c_interface_generate_write_preamble(pObj,  addr, &preamble);

	apiRetStatus = CyU3PI2cTransmitBytes (&preamble, pBuf, size, 0);
	if(apiRetStatus == CY_U3P_SUCCESS)
	{
		apiRetStatus = CyU3PI2cWaitForAck(&preamble, 0);
	}

	return apiRetStatus;
}

static int i2c_interface_read_burst(void *p, uint32_t addr, uint8_t *pBuf, uint32_t size)
{
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    CyU3PI2cPreamble_t preamble;
	I2COBJ_t *pObj = (I2COBJ_t *)p;
	i2c_interface_generate_read_preamble(pObj, addr, &preamble);

    apiRetStatus = CyU3PI2cReceiveBytes (&preamble, pBuf, size, 3);

    CyU3PThreadSleep (1);

    return apiRetStatus;
}


static int i2c_interface_generate_write_preamble(I2COBJ_t *pObj,  uint32_t addr, CyU3PI2cPreamble_t *p_preamble)
{
	p_preamble->buffer[0] = ((pObj->slave_addr << 1) & 0xFE);  //һλǶд־λ0д
	if(pObj->addr_len == 0)
	{

	}
	else if(pObj->addr_len == 1)
	{
		p_preamble->buffer[1] = UINT32BYTE0(addr);
	}
	else if(pObj->addr_len == 2)
	{
		p_preamble->buffer[1] = UINT32BYTE1(addr);
		p_preamble->buffer[2] = UINT32BYTE0(addr);

	}
	else if (pObj->addr_len == 3)
	{
		p_preamble->buffer[1] = UINT32BYTE2(addr);
		p_preamble->buffer[2] = UINT32BYTE1(addr);
		p_preamble->buffer[3] = UINT32BYTE0(addr);
	}
	else
	{
		p_preamble->buffer[1] = UINT32BYTE3(addr);
		p_preamble->buffer[2] = UINT32BYTE2(addr);
		p_preamble->buffer[3] = UINT32BYTE1(addr);
		p_preamble->buffer[4] = UINT32BYTE0(addr);
	}
	p_preamble->length    = pObj->addr_len + 1;
	p_preamble->ctrlMask  = 0x0000;
	return 0;
}

static int i2c_interface_generate_read_preamble(I2COBJ_t *pObj, uint32_t addr, CyU3PI2cPreamble_t *p_preamble)
{
	p_preamble->buffer[0] = (pObj->slave_addr << 1) & 0xFE;

	if(pObj->addr_len == 0)
	{
		p_preamble->buffer[1] = (pObj->slave_addr << 1) | 0x01;
		p_preamble->length = 2;
		/*  Send start bit after first byte of preamble. */
		p_preamble->ctrlMask = 0x0001;
	}
	if(pObj->addr_len == 1)
	{
		p_preamble->buffer[1] = UINT32BYTE0(addr);
		p_preamble->buffer[2] = (pObj->slave_addr << 1) | 0x01;
		p_preamble->length = 3;
		/*  Send start bit after second byte of preamble. */
		p_preamble->ctrlMask = 0x0002;
	}
	else if(pObj->addr_len == 2)
	{
		p_preamble->buffer[1] = UINT32BYTE1(addr);
		p_preamble->buffer[2] = UINT32BYTE0(addr);
		p_preamble->buffer[3] = (pObj->slave_addr << 1) | 0x01;
		p_preamble->length = 4;
		/*  Send start bit after third byte of preamble. */
		p_preamble->ctrlMask = 0x0004;

	}
	else if (pObj->addr_len == 3)
	{
		p_preamble->buffer[1] = UINT32BYTE2(addr);
		p_preamble->buffer[2] = UINT32BYTE1(addr);
		p_preamble->buffer[3] = UINT32BYTE0(addr);
		p_preamble->buffer[4] = (pObj->slave_addr << 1) | 0x01;
		p_preamble->length = 5;
		/*  Send start bit after fourth byte of preamble. */
		p_preamble->ctrlMask = 0x0008;
	}
	else
	{
		p_preamble->buffer[1] = UINT32BYTE3(addr);
		p_preamble->buffer[2] = UINT32BYTE2(addr);
		p_preamble->buffer[3] = UINT32BYTE1(addr);
		p_preamble->buffer[4] = UINT32BYTE0(addr);
		p_preamble->buffer[5] = (pObj->slave_addr << 1) | 0x01;
		p_preamble->length = 6;
		/*  Send start bit after fourth byte of preamble. */
		p_preamble->ctrlMask = 0x0010;
	}

	return 0;
}
